/**
 * @fileOverview
 * @name user.js
 * @author Wagner Müller Gegler, Roberto Oppermann Cordoni
 */

/**
 * @class UserController
 * @param {Object} $http - AngularJS service for HTTP communication.
 * @param {Object} $log - AngularJS service for logging.
 * @property {String} password - Password text.
 * @property {String} confirm_password - Confirmation password text.
 */
angular.module('ag561app').controller('UserController', ['$http', '$log',
                                   function($http, $log) {
  var url_login = '/api/config/login';
  var ctrl = this;
  ctrl.loading = false;
  ctrl.state = 'view';
  ctrl.user = null;

  /**
   * Clear alert message from web page, but do this only
   * if we aren't with some error. We just want to erase
   * the message of danger if all the variables are correct.
   *
   * @function clear_alert_messages
   * @memberof NetworkController
   */
  ctrl.clear_alert_messages = function() {
    ctrl.alert_danger = false;
    ctrl.alert_success = false;
  };

  /**
   * Start user 'edit' mode
   * Save user object passed as parameter
   */
  ctrl.user_edit = function(user) {
    /* Load form variables */
    ctrl.user = user;
    ctrl.password = "";
    ctrl.confirm_password = "";
    if (user) {
      ctrl.username = ctrl.user.name;
      ctrl.privilege_level = ctrl.user.privilege_level;
      ctrl.concurrent_login = ctrl.user.concurrent_login;
      ctrl.valid_config = true;
    } else {
      ctrl.username = "";
      ctrl.privilege_level = "user";
      ctrl.concurrent_login = "yes";
      ctrl.valid_config = false;
    }

    /* Go to 'edit' mode */
    ctrl.state = 'edit';
  };

  /**
   * Cancel 'edit' mode, reset variables and go back to 'view' mode
   */
  ctrl.user_cancel = function() {
    ctrl.user = null;
    ctrl.clear_alert_messages();
    ctrl.alert_warning = '';
    ctrl.state = 'view';
  };

  /* Add new user, save user config and go back to 'view' mode */
  ctrl.user_save = function() {
    if (!ctrl.user) {
      /* Add new user */
      ctrl.user = { name : ctrl.username };
      ctrl.users.push(ctrl.user);
    }

    /* Save user config */
    ctrl.user.privilege_level = ctrl.privilege_level;
    ctrl.user.concurrent_login = ctrl.concurrent_login;
    if (ctrl.password.length > 0) {
      ctrl.user.password = ctrl.password;
    }

    /* Go back to 'view' mode */
    ctrl.user = null;
    ctrl.state = 'view';
  };

  /**
   * Remove a user from users list.
   * @function user_delete
   * @param {number} index      The user object's index in users list.
   * @memberof UserController
   */
  ctrl.user_delete = function(index) {
    /* Cannot delete user admin*/
    if (ctrl.users[index].name == 'admin')
      return;

    ctrl.users.splice(index, 1);
  };

  /**
   * Check if a user exists
   */
  ctrl.user_exists = function(name) {
    for (var i = 0; i < ctrl.users.length; i++) {
      if (ctrl.users[i].name == name) {
        return true;
      }
    }
    return false;
  }

  /**
   * Check user config
   */
  ctrl.check_config = function() {
    /* Check if username is valid for a new user */
    if (!ctrl.user && !ctrl.check_username()) {
      ctrl.valid_config = false;
    /* Check if password is valid */
    } else if (ctrl.check_password()) {
      ctrl.valid_config = false;
    }
    ctrl.valid_config = true;
  };

  /**
   * Check if new user name is valid
   */
  ctrl.check_username = function() {
    /* Check if username is nor too short and has valid characters */
    if (ctrl.username.length == 0 || !ctrl.username.match(/^\w+$/)) {
      ctrl.alert_warning = 'Invalid username.';
    /* Check if username is reserved */
    } else if (ctrl.reserved_users.indexOf(ctrl.username) >= 0) {
      ctrl.alert_warning = 'Username is reserved.';
    /* Check if new user already exists */
    } else if (ctrl.user_exists(ctrl.username)) {
      ctrl.alert_warning = 'User already exists.';
    } else {
      /* Username is valid */
      ctrl.alert_warning = '';
      return true;
    }
    /* Username is invalid */
    return false;
  };

  /**
   * Check if the password fields have more than 4 chars, and if they match.
   * @function check_password
   * @memberof UserController
   */
  ctrl.check_password = function() {
    /* Check if password is not too short (password change may be empty for an existing user */
    if (ctrl.password.length < 4 && (!ctrl.user || ctrl.password.length > 0)) {
      ctrl.alert_warning = 'Password too short.';
    /* Check if password has valid characters */
    } else if (!ctrl.password.match(/^[\40-\177]*$/)) {
      ctrl.alert_warning = 'Invalid password.';
    /* Check if password and confirmation match */
    } else if (ctrl.password != ctrl.confirm_password) {
      ctrl.alert_warning = 'Passwords do not match.';
    } else {
      /* Password is valid */
      ctrl.alert_warning = '';
      return true;
    }
    /* Password is invalid */
    return false;
  };

  /**
   * Get login users.
   *
   * @function get
   * @memberof UserController
   */
  ctrl.get = function() {
    $http({
      method: 'GET',
      url: url_login
      }).then(function(response) {
        // First function handles success
        ctrl.users = response.data.users;
        ctrl.reserved_users = response.data.reserved_users;
      }, function(response) {
        // Second function handles error
        ctrl.error = response.data;
      });
  };

  /**
   * Apply changes for system users.
   * @function save_and_apply
   * @memberof UserController
   */
  ctrl.save_and_apply = function() {
    ctrl.loading = true;
    var data = {'users': ctrl.users}
    $http({
      method: 'PUT',
      url: url_login,
      data: data,
      }).then(function(response) {
        // First function handles success
        ctrl.loading = false;
        ctrl.alert_success = true;
      }, function(response) {
        // Second function handles error
        ctrl.loading = false;
        ctrl.alert_danger = true;
      });
  };

  ctrl.get();

}]);
